#pragma once

#include <string>
#include <vector>

#include <Poco/Process.h>
#include <Poco/SharedPtr.h>
#include <Poco/PipeStream.h>

#include "Constants.h"

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_CPP_API __declspec(dllexport)
#else
#define FASPSTREAM_CPP_API __declspec(dllimport)
#endif

#else // NOT WIN32
#define FASPSTREAM_CPP_API
#endif

/**
 * Use the FaspInputStreamFromFile class to receive file data streamed
 * from a file located on an Aspera server.  Note this class does
 * not inherit from any of the C++ std::ios stream classes, and
 * requires the user to implement a read() loop to receive the data.
 */
class FaspInputStreamFromFile
{

private:
    Poco::SharedPtr<Poco::ProcessHandle> _ph;
    Poco::Pipe _outPipe;
    Poco::Pipe _errPipe;

    int _udpPort;
    int _tcpPort;
    int _maxMemory;
    std::string _ascpPath;
    std::string _sshKeyFilePath;
    std::string _sourceFilePath;
    std::string _username;
    std::string _password;
    std::string _hostname;
    long _datagramSize;
    long _chunkSize;
    long _targetRate;
    long _minRate;
    FASPSTREAM_POLICY_TYPE _policy;
    bool _eof;
    std::string _license;

public:
     /**
     * This method can be used to generate the command line arguments
     * with which the faspstream child process will be invoked, it can be
     * useful for debugging purposes but is not normally necessary.
     */
    std::vector<std::string> getCommandline() const;

    FASPSTREAM_CPP_API FaspInputStreamFromFile();
    FASPSTREAM_CPP_API ~FaspInputStreamFromFile();
    FASPSTREAM_CPP_API void setAscpPath(const std::string& ascpPath);
    FASPSTREAM_CPP_API std::string getAscpPath() const;
    FASPSTREAM_CPP_API void setSshKeyFilePath(
        const std::string& sshKeyFilePath);
    FASPSTREAM_CPP_API std::string getSshKeyFilePath() const;
    FASPSTREAM_CPP_API void setSourceFilePath(
        const std::string& sourceFilePath);
    FASPSTREAM_CPP_API std::string getSourceFilePath() const;
    FASPSTREAM_CPP_API void setUsername(const std::string& username);
    FASPSTREAM_CPP_API std::string getUsername() const;
    FASPSTREAM_CPP_API void setPassword(const std::string& password);
    FASPSTREAM_CPP_API std::string getPassword() const;
    FASPSTREAM_CPP_API void setHostname(const std::string& hostname);
    FASPSTREAM_CPP_API std::string getHostname() const;
    FASPSTREAM_CPP_API void setDatagramSize(const long datagramSize );
    FASPSTREAM_CPP_API long getDatagramSize() const;
    FASPSTREAM_CPP_API void setChunkSize(const long chunkSize);
    FASPSTREAM_CPP_API long getChunkSize() const;
    FASPSTREAM_CPP_API void setTargetRate(const long minRate);
    FASPSTREAM_CPP_API long getTargetRate() const;
    FASPSTREAM_CPP_API void setMinRate(const long minRate);
    FASPSTREAM_CPP_API long getMinRate() const;
    FASPSTREAM_CPP_API void setPolicy(FASPSTREAM_POLICY_TYPE ascpPath);
    FASPSTREAM_CPP_API FASPSTREAM_POLICY_TYPE getPolicy() const;
    FASPSTREAM_CPP_API bool startReceiver();
    FASPSTREAM_CPP_API int getTcpPort() const;
    FASPSTREAM_CPP_API void setTcpPort(int udpPort);
    FASPSTREAM_CPP_API int getUdpPort() const;
    FASPSTREAM_CPP_API void setUdpPort(int udpPort);
    FASPSTREAM_CPP_API int getMaxMemory() const;
    FASPSTREAM_CPP_API void setMaxMemory(int maxMem);
    FASPSTREAM_CPP_API std::string getLogDirectory() const;
    FASPSTREAM_CPP_API int read(char* buffer, int offset, int length);
    FASPSTREAM_CPP_API std::string close();
    FASPSTREAM_CPP_API void forceClose();
    FASPSTREAM_CPP_API bool eof();
    FASPSTREAM_CPP_API void setLicense(std::string commandToExecute);
    FASPSTREAM_CPP_API std::string getLicense() const;
};
