// SenderExample.cpp : Example of how to send a FaspOutputStream (the contents of a file)
//

#include <iostream>
#include <fstream>

#include <Constants.h>
#include <FaspOutputStreamToFile.h>
#include <FaspStreamException.h>

int main(int argc, char* argv[])
{
    char *data = new char[Constants::TWO_Mb];

	std::string fileToSend("(UNSPECIFIED FILE)");

    try {

        if (argc < 2) {
            std::cout << "Usage:  SenderExample file_to_send\n";
            return 1;
        }
        
		fileToSend = argv[1];

        FaspOutputStreamToFile faspOutputStream;
	faspOutputStream.setAscpPath("./ascp");
        faspOutputStream.setHostname("demo.asperasoft.com");
        faspOutputStream.setUsername("aspera");
        faspOutputStream.setPassword("demoaspera");

        faspOutputStream.setDestinationDirectory("/Upload");
        faspOutputStream.setDestinationFilename("StreamTestFile");

        //Set the port to connect on
        // faspOutputStream.setPort(33445);

        //Set Target Rate to 1Gb
        //faspOutputStream.setRateTarget(1000000);

	//Read from source file and send to receiver
	std::ifstream in(fileToSend.c_str(), std::ios::in | std::ios::ate | std::ios::binary);
	in.exceptions( std::ifstream::badbit );

	if (!in.is_open())
		throw std::ifstream::failure(("error while opening file " + fileToSend).c_str());

        long fileLength = in.tellg(); // get file size
        in.seekg(0, std::ios::beg); //reset stream to beginning

        //Connect to Receiver
        if (faspOutputStream.connect(fileLength))
        {
              while(in)
		      {
		          in.read(data, Constants::TWO_Mb);
		          faspOutputStream.write(data, static_cast<int>(in.gcount()));
		      }

		      //Close the sender stream
		      std::cout << faspOutputStream.close() << std::endl;
	    }

    }  catch (FaspStreamException &e) {
        std::cout << "A faspstream exception occurred: ";
	std::cout << e.what();
	std::cout << "Debug details:  " << e.getDebugDetails() << std::endl;
    }  catch (std::ifstream::failure &e) {
        std::cout << "A file I/O exception occurred with: " << fileToSend.c_str() << std::endl;
        std::cout << e.what() << std::endl;
    }  catch (std::exception &e) {
        std::cout << "A standard exception occurred:" << std::endl;
        std::cout << e.what();
    } catch (...) {
        std::cout << "An unknown exception ocurred!";
        std::cout << "(No further information is available)";
    }

    delete [] data;

    return 0;
}

